﻿Public Class Form1

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Declarations ''''''''''''''''''''''''''''''''''''''''''''''''''''
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Dim uart_in, byte_cnt As Byte
    Dim temp, time, bytes_count As UInt16
    Dim logpath As String

    ' This delegate enables asynchronous calls for setting
    ' the text property on a TextBox control.
    Delegate Sub SetTextCallback(ByVal [text] As String)
    Delegate Sub SetNumberCallback(ByVal value As UInt16)

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' The next four subrutines handle connecting to the serial port '''
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Private Sub connect_button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles connect_button.Click
        ' Check that save location has been chosen
        Try
            If logpath.Length < 2 Then
                MessageBox.Show("Select a valid location for your log file first.")
            Else
                Try
                    SerialPort1.Open()
                    disconnect_button.Enabled = True
                    connect_button.Enabled = False
                    save_button.Enabled = False
                    ComboBox1.Enabled = False
                    Timer1.Enabled = True
                    Reset_stat()
                    My.Computer.FileSystem.WriteAllText(logpath, "Relative Time (s)" + vbTab + "Counts detected" + Environment.NewLine, True)
                Catch unauthorized As UnauthorizedAccessException
                    MessageBox.Show("This program is forbidden access to the COM port. It may be in use by another application, or blocked for security reasons.")
                Catch exception As IO.IOException
                    MessageBox.Show("Selected COM port wasn't found on this machine, or baudrate is out of range.")
                Catch arg As InvalidOperationException
                    MessageBox.Show("Selected COM port is already open.")
                End Try
            End If
        Catch ex As System.NullReferenceException
            MessageBox.Show("Select a location for your log file first.")
        End Try
    End Sub

    Private Sub disconnect_button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles disconnect_button.Click
        Try
            connect_button.Enabled = True
            disconnect_button.Enabled = False
            save_button.Enabled = True
            ComboBox1.Enabled = True
            Timer1.Enabled = False
            SerialPort1.Close()
        Catch invalid As InvalidOperationException
            MessageBox.Show("Selected COM port is already closed.")
        End Try
    End Sub

    Private Sub ComboBox1_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBox1.SelectedIndexChanged
        SerialPort1.PortName = ComboBox1.Text
    End Sub

    Private Sub Reset_stat()
        temp = 0
        byte_cnt = 0
        time = 0
        bytes_count = 0
    End Sub

    Private Sub LinkLabel1_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel1.LinkClicked
        System.Diagnostics.Process.Start("http://www.google.com/search?hl=en&q=uzzors2k")
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        time = time + 1
        time_box.Text = time.ToString
    End Sub

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' The next four rutines receive serial bytes and display them '''''
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Private Sub SerialPort1_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        ' Check if there are bytes in the buffer, if so, flush them out
        While Not (SerialPort1.BytesToRead = 0)
            uart_in = SerialPort1.ReadByte
            'My.Computer.FileSystem.WriteAllText("C:\Users\Eirik\Desktop\geiger_raw_log.txt", uart_in.ToString + ";", True)
            inputprocess(uart_in)
        End While

    End Sub

    ' This method is passed in to the SetTextCallBack delegate
    ' to set the Text property of textBox1.
    Private Sub SetText1(ByVal [text] As String)
        counts_box.Text = [text]
    End Sub

    Private Sub SetText2(ByVal [text] As String)
        bytes_log_box.Text = [text]
    End Sub

    Private Sub SetChart1(ByVal value As UInt16)
        Chart1.Series("Series1").Points.AddY(value)
    End Sub

    Private Sub inputprocess(ByVal [data] As Byte)
        ' Increment byte counter
        byte_cnt = byte_cnt + 1

        If byte_cnt = 4 Then
            ' MSB of data
            temp = data
            temp = (temp << 8) And 32512

        ElseIf byte_cnt = 5 Then
            ' LSB of data
            temp = temp Or data

        ElseIf ((byte_cnt < 4) And (Not (data = 255))) Or ((byte_cnt > 5) And (Not (data = 128))) Then
            ' Invalid byte received
            byte_cnt = 0
            temp = 0

        ElseIf (data = 128) And (byte_cnt = 6) Then
            ' End byte received
            byte_cnt = 0
            bytes_count = bytes_count + 1
            If bytes_log_box.InvokeRequired Then
                Dim d As New SetTextCallback(AddressOf SetText2)
                Me.Invoke(d, New Object() {bytes_count.ToString})
            Else
                bytes_log_box.Text = bytes_count.ToString
            End If

            ' Valid byte sequence received, count value can be logged
            My.Computer.FileSystem.WriteAllText(logpath, time.ToString + vbTab + temp.ToString + Environment.NewLine, True)

            ' And displayed
            If counts_box.InvokeRequired Then
                Dim d As New SetTextCallback(AddressOf SetText1)
                Me.Invoke(d, New Object() {temp.ToString})
            Else
                counts_box.Text = temp.ToString
            End If

            ' And graphed
            If Chart1.InvokeRequired Then
                Dim d As New SetNumberCallback(AddressOf SetChart1)
                Me.Invoke(d, New Object() {temp})
            Else
                Chart1.Series("Series1").Points.AddY(temp)
            End If

            temp = 0
        End If
    End Sub

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' The next rutine allows the user to select a location for the ''''
    ' log file                                                     ''''
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    ' Displays a SaveFileDialog 
    Private Sub save_button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles save_button.Click
        Dim saveFileDialog1 As New SaveFileDialog()
        saveFileDialog1.Filter = "Text File|*.txt"
        saveFileDialog1.Title = "Select Location for Log File"
        saveFileDialog1.ShowDialog()

        ' If the file name is not an empty string open it for saving.
        If saveFileDialog1.FileName <> "" Then
            ' Saves the text file via a FileStream created by the OpenFile method.
            Dim fs As System.IO.FileStream = CType _
               (saveFileDialog1.OpenFile(), System.IO.FileStream)

            ' Saves the file location
            logpath = saveFileDialog1.FileName
            location_box.Text = logpath

            ' Closes File stream
            fs.Close()
        End If
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

    End Sub
End Class
